import java.net.*;
import java.io.*;

public class ClientSocketHandler {

	private Socket socket;
	private boolean connected = false;
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private String agentName = null;
	private String serverName = null;
	private int serverPort;
	private Object lastMessage = null;
	private Object lastResponse = null;
	
	ClientSocketHandler(String agentName, String serverName, int serverPort, int startGame){
		this.agentName = agentName;
		this.serverName = serverName;
		this.serverPort = serverPort;
		
		connect(startGame);	
	}
	
	private void connect(int startGame){//connect for first time
		connect("connect " + startGame);
	}

	private void connect(String status){
		while (!connected){
			try {
				socket = new Socket(serverName, serverPort);
				socket.setSoTimeout(60000);
				out = new ObjectOutputStream(socket.getOutputStream());
				in = new ObjectInputStream(socket.getInputStream());
				//first message should be agent's name
				out.writeObject(agentName);
				out.flush();
				//second message should be status - connect, reconnect, or connect (game#)
				out.writeObject(status);
				out.flush();
				connected = true;
			} catch (Exception e){
				System.out.println(e.getMessage());
				System.out.println("Unable to connect, will try again");
				try{Thread.sleep(5000);} catch (Exception e2){};
			}
		}
	}
	
	public void writeObject(Object o){
		boolean success = false;
		while (!success){
			try {
				lastResponse = o;
				out.writeObject(o);
				out.flush();
				success = true;
			} catch (Exception e){
				System.out.println(e.getMessage() + "\nReconnecting.");
				connected = false;
				connect("reconnect");
			}
		}
	}

	public Object readObject(){
		Object o = null;
		while (o == null){
			try { 
				o = in.readObject();
				if (lastMessage != null && 
						(lastMessage instanceof NewGameMessage && o instanceof NewGameMessage ||
						lastMessage instanceof DailyInfo && o instanceof DailyInfo && 
							((DailyInfo)o).date == ((DailyInfo)lastMessage).date) 		
						){//server resent message after reconnect
					out.writeObject(lastResponse);
					o = in.readObject();
				}
				lastMessage = o;
				if (o instanceof NewGameMessage) //new game, so safe to reset and allow GC
					out.reset();
			} catch (Exception e){
				System.out.println(e.getMessage() + "\nReconnecting.");
				connected = false;
				connect("reconnect");
			}
		}
		return o;
	}

	public void close(){
		try {
			socket.close();
		} catch (Exception e){e.printStackTrace();}
	}
	
}
