import java.io.*;
import se.sics.isl.util.ArgumentManager;

public class PredictionClient {
    public static void main(String[] args) throws IOException {
    	
    	ArgumentManager config = new ArgumentManager("PredictionClient", args);
		config.addOption("server", "name", "server to connect to");
		config.addOption("name", "name", "team name");
		config.addOption("game", "number", "game number to start on (default 1)");
		config.addOption("predictor", "predictorClass", "set the class to handle predictions");
		config.addHelp("h", "show this help message");
		config.addHelp("help");
		config.validateArguments();

		config.finishArguments();
    	
    	String server = config.getProperty("server");
    	String name = config.getProperty("name");
    	String predictorClass = config.getProperty("predictor");
    	int startGame;
   		startGame = config.getPropertyAsInt("game", 1);
    	
    	int port = 7070;
    	SCMPredictor predictor = null;
    	
    	ClientSocketHandler sh = new ClientSocketHandler(name, server, port, startGame);
    	
    	try{
			predictor = (SCMPredictor) Class.forName(predictorClass).newInstance();
		}
		catch (Exception e) {
			System.out.println("Error creating instance of predictor class: " + predictorClass);
			e.printStackTrace();	
		}
		
		boolean done = false;
		while (!done){
			Object o = sh.readObject();
			if (o instanceof NewGameMessage){
				NewGameMessage n = (NewGameMessage)o;
				if (n.finished)//done with all games
					done = true;
				else{
					predictor.nextGame(n.newGroup);
					//System.out.println("Starting predictions for a new game.");
				}
				sh.writeObject("ACK");
			}
			else if (o instanceof DailyInfo){
				DailyInfo info = (DailyInfo) o;
				predictor.nextDay(info.messagesIn, info.messagesOut, info.predictions);

				//remove unneeded info before returning predictions to save space
				info.predictions.futureComponentRFQList = null;
				
				sh.writeObject(info.predictions);
			}
			else if (o instanceof String){
				System.out.print(o);
				sh.writeObject("ACK");
			}
		}
    }
}

