import java.net.*;
import java.io.*;
import java.util.*;

import se.sics.isl.util.ArgumentManager;

public class PredictionServer {
	static ArgumentManager config;
	static Hashtable sessions = new Hashtable();
	
	public static void main(String[] args) throws IOException {
        
		config = new ArgumentManager("PredictionServer", args);
		config.addOption("gamelist", "file", "file with list of games to use");
		config.addOption("agentName", "name", "the name of the agent to use from the games");
		config.addOption("verbose", "use verbose output");
		config.addHelp("h", "show this help message");
		config.addHelp("help");
		config.validateArguments();

		config.finishArguments();

		
		run();
    }

	private static void run() throws IOException {
		ServerSocket serverSocket = null;
        boolean listening = true;

        try {
            serverSocket = new ServerSocket(7070);
        } catch (IOException e) {
            System.err.println("Could not listen on port: 7070.");
            System.exit(-1);
        }

        while (listening){
        	Socket socket = serverSocket.accept();
        	ServerSocketHandler sh = new ServerSocketHandler(socket);
        	String name = sh.getName(); 
        	if (name != null){
        		if (sessions.containsKey(name)){ //agent is reconnecting, resume session
        			ServerSocketHandler oldsh = (ServerSocketHandler) sessions.get(name);
        			if (oldsh.isClosed() || sh.isRestart()){//previous session completed, remove socket handler

        				sessions.remove(name);
        			}
        			else{
        				System.out.println("resuming session for " + name);
        				sh.restoreSocketHandler(oldsh);
        			}
        		}
        		if (!sessions.containsKey(name)) {//new session
            		PredictionServerThread t = new PredictionServerThread(sh, config, name);
            		sessions.put(name, sh);
            		System.out.println("beginning session for " + name);
            		t.start();
        		}
        	}
        }
        serverSocket.close();
		
	}
}

