import java.net.*;
import java.io.*;

public class ServerSocketHandler {

	private Socket socket;
	private boolean connected = false;
	private Object lock = new Object();
	private ObjectInputStream in;
	private ObjectOutputStream out;
	private String agentName = null;
	private boolean closed = false;
	private boolean restart = false;
	private int startGame = 1;
	
	ServerSocketHandler(Socket socket){
		this.socket = socket;
		try {
			in = new ObjectInputStream(socket.getInputStream());
			out = new ObjectOutputStream(socket.getOutputStream());
			//first message should be agent's name
			agentName = (String) in.readObject();
			connected = true;
			//second message should be status - connect or reconnect, or connect game#
			String status = (String) in.readObject();
			restart = (status.startsWith("connect"));
			if (restart){
				String[] s = status.split(" ");
				if (s.length == 2){
					startGame = Integer.parseInt(s[1]);
				}
			}
				
		} catch (Exception e){e.printStackTrace();}	
	}
	
	public String getName(){
		return agentName;
	}
	
	public Object writeAndRead(Object o){
		boolean success = false;
		Object reply = null;
		connected = false;
		while (!success){
			try {
				if (o instanceof NewGameMessage) //new game, so safe to reset and allow GC
					out.reset();
				out.writeObject(o);
				out.flush();
				reply = in.readObject();
				if (reply == null)
					throw new Exception("");
				success = true;
			} catch (Exception e){
				System.out.println("Lost connection to " + agentName);
				synchronized(lock){
					if (! connected)
						try{lock.wait();}catch(Exception e2){
						}
					}
				System.out.println("Reconnected to " + agentName);	
			}
		}
		connected = true;
		return reply;
	}
		
	public void close(){
		try {
			socket.close();
		} catch (Exception e){e.printStackTrace();}
		closed = true;
	}
	
	private void reconnect(Socket socket, ObjectInputStream in, ObjectOutputStream out){
		synchronized(lock){
			try {
				if (this.in != null)
					this.in.close();
				if (this.out != null)
					this.out.close();
			} catch (Exception e){/*e.printStackTrace();*/}
			
			this.socket = socket;
			this.in = in;
			this.out = out;
			connected = true;

			lock.notify();
		}
	}
	
	public void restoreSocketHandler(ServerSocketHandler oldsh){
		oldsh.reconnect(socket, in, out);
	}
	
	public boolean isClosed(){
		return closed;
	}
	
	public boolean isRestart(){
		return restart;
	}

	public int getStartGame(){
		return startGame;
	}
	
}
